@@ -7,39 +7,47 @@
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Application support
 
-$Id: Application.py,v 1.191.2.13 2004/11/29 20:38:50 shh Exp $
+$Id: Application.py 41579 2006-02-08 15:45:05Z anguenot $
 """
-import Globals,Folder,os,sys,App.Product, App.ProductRegistry, misc_
-import time, traceback, os,  Products
-from DateTime import DateTime
-from AccessControl.User import UserFolder
-from App.ApplicationManager import ApplicationManager
-from webdav.NullResource import NullResource
-from FindSupport import FindSupport
+
+import os, sys, traceback
 from cgi import escape
-from urllib import quote
 from StringIO import StringIO
-from AccessControl.PermissionRole import PermissionRole
-from App.ProductContext import ProductContext
-from misc_ import Misc_
-import ZDOM
-from zLOG import LOG, ERROR, WARNING, INFO
-from zExceptions import Redirect as RedirectException, Forbidden
-from HelpSys.HelpSys import HelpSys
+from warnings import warn
+
+import Globals, Products, App.Product, App.ProductRegistry
+import transaction
+from AccessControl.User import UserFolder
 from Acquisition import aq_base
-from App.Product import doInstall
+from App.ApplicationManager import ApplicationManager
 from App.config import getConfiguration
+from App.Product import doInstall
+from App.ProductContext import ProductContext
+from DateTime import DateTime
+from HelpSys.HelpSys import HelpSys
+from webdav.NullResource import NullResource
+from zExceptions import Redirect as RedirectException, Forbidden
+from zLOG import LOG, ERROR, WARNING, INFO
+
+import Folder
+import misc_
+import ZDOM
+from FindSupport import FindSupport
+from misc_ import Misc_
+
 
 class Application(Globals.ApplicationDefaultPermissions,
                   ZDOM.Root, Folder.Folder,
                   App.ProductRegistry.ProductRegistry, FindSupport):
+
     """Top-level system object"""
-    title    ='Zope'
+
+    title ='Zope'
     #__roles__=['Manager', 'Anonymous']
     __defined_roles__=('Manager','Anonymous','Owner')
     web__form__method='GET'
@@ -68,9 +76,6 @@ class Application(Globals.ApplicationDefaultPermissions,
 
     _initializer_registry = None
 
-    def title_and_id(self): return self.title
-    def title_or_id(self): return self.title
-
     def __init__(self):
         # Initialize users
         uf=UserFolder()
@@ -81,20 +86,28 @@ class Application(Globals.ApplicationDefaultPermissions,
         cpl=ApplicationManager()
         cpl._init()
         self._setObject('Control_Panel', cpl)
-        get_transaction().note("Created Zope Application")
+        transaction.get().note("Created Zope Application")
 
     def id(self):
         try:    return self.REQUEST['SCRIPT_NAME'][1:]
         except: return self.title
 
-    def __class_init__(self): Globals.default__class_init__(self)
+    def title_and_id(self):
+        return self.title
 
-    def PrincipiaRedirect(self,destination,URL1):
+    def title_or_id(self):
+        return self.title
+
+    def __class_init__(self):
+        Globals.default__class_init__(self)
+
+    def PrincipiaRedirect(self, destination, URL1):
         """Utility function to allow user-controlled redirects"""
         if destination.find('//') >= 0:
             raise RedirectException, destination
         raise RedirectException, ("%s/%s" % (URL1, destination))
-    Redirect=ZopeRedirect=PrincipiaRedirect
+
+    Redirect = ZopeRedirect = PrincipiaRedirect
 
     def __bobo_traverse__(self, REQUEST, name=None):
 
@@ -115,7 +128,8 @@ class Application(Globals.ApplicationDefaultPermissions,
     def PrincipiaTime(self, *args):
         """Utility function to return current date/time"""
         return apply(DateTime, args)
-    ZopeTime=PrincipiaTime
+
+    ZopeTime = PrincipiaTime
 
     ZopeAttributionButton__roles__=None
     def ZopeAttributionButton(self):
@@ -137,10 +151,11 @@ class Application(Globals.ApplicationDefaultPermissions,
         raise Forbidden, 'This resource cannot be moved.'
 
     test_url___allow_groups__=None
-    test_url=ZopeAttributionButton
+    test_url = ZopeAttributionButton
 
     def absolute_url(self, relative=0):
-        '''The absolute URL of the root object is BASE1 or "/".'''
+        """The absolute URL of the root object is BASE1 or "/".
+        """
         if relative: return ''
         try:
             # Take advantage of computed URL cache
@@ -149,26 +164,32 @@ class Application(Globals.ApplicationDefaultPermissions,
             return '/'
 
     def absolute_url_path(self):
-        '''The absolute URL path of the root object is BASEPATH1 or "/".'''
+        """The absolute URL path of the root object is BASEPATH1 or "/".
+        """
         try:
             return self.REQUEST['BASEPATH1'] or '/'
         except (AttributeError, KeyError):
             return '/'
 
     def virtual_url_path(self):
-        '''The virtual URL path of the root object is empty.'''
+        """The virtual URL path of the root object is empty.
+        """
         return ''
 
+    def getPhysicalRoot(self):
+        return self
+
     def getPhysicalPath(self):
-        '''Returns a path that can be used to access this object again
-        later, for example in a copy/paste operation.  Designed to
-        be used with getPhysicalRoot().
-        '''
+        """Get the physical path of the object.
+
+        Returns a path (an immutable sequence of strings) that can be used to
+        access this object again later, for example in a copy/paste operation.
+        getPhysicalRoot() and getPhysicalPath() are designed to operate
+        together.
+        """
         # We're at the base of the path.
         return ('',)
 
-    def getPhysicalRoot(self): return self
-
     fixupZClassDependencies__roles__=()
     def fixupZClassDependencies(self, rebuild=0):
         # Note that callers should not catch exceptions from this method
@@ -256,7 +277,9 @@ class Application(Globals.ApplicationDefaultPermissions,
             reg = {}
         return reg.get(flag)
 
+
 class Expired(Globals.Persistent):
+
     icon='p_/broken'
 
     def __setstate__(self, s={}):
@@ -277,7 +300,9 @@ def initialize(app):
     initializer = AppInitializer(app)
     initializer.initialize()
 
+
 class AppInitializer:
+
     """ Initialze an Application object (called at startup) """
 
     def __init__(self, app):
@@ -288,8 +313,8 @@ class AppInitializer:
         return self.app[0]
 
     def commit(self, note):
-        get_transaction().note(note)
-        get_transaction().commit()
+        transaction.get().note(note)
+        transaction.commit()
 
     def initialize(self):
         app = self.getApp()
@@ -324,10 +349,10 @@ class AppInitializer:
 
     def install_tempfolder_and_sdc(self):
         app = self.getApp()
-        from Products.ZODBMountPoint.MountedObject \
-             import manage_addMounts, MountedObject
-        from Products.ZODBMountPoint.MountedObject \
-             import getConfiguration as getDBTabConfiguration
+        from Products.ZODBMountPoint.MountedObject import manage_addMounts,\
+             MountedObject
+        from Products.ZODBMountPoint.MountedObject import getConfiguration as \
+             getDBTabConfiguration
 
         dbtab_config = getDBTabConfiguration()
 
@@ -379,8 +404,8 @@ class AppInitializer:
             default_period_secs = 20
             default_timeout_mins = 20
 
-            limit = (getattr(config, 'maximum_number_of_session_objects', None)
-                     or default_limit)
+            limit = getattr(config, 'maximum_number_of_session_objects',
+                            default_limit)
             timeout_spec = getattr(config, 'session_timeout_minutes',
                                    default_timeout_mins)
             period_spec = getattr(config, 'session_resolution_seconds',
@@ -532,13 +557,13 @@ class AppInitializer:
                 did_fixups=1
                 LOG('Zope', INFO,
                     'The global ZClass registry has successfully been rebuilt.')
-                get_transaction().note('Rebuilt global product registry')
-                get_transaction().commit()
+                transaction.get().note('Rebuilt global product registry')
+                transaction.commit()
         except:
             bad_things=1
             LOG('Zope', ERROR, 'The attempt to rebuild the registry failed.',
                 error=sys.exc_info())
-            get_transaction().abort()
+            transaction.abort()
 
         # Now we need to see if any (disk-based) products were installed
         # during intialization. If so (and the registry has no errors),
@@ -565,7 +590,7 @@ class AppInitializer:
                         ('Attempt to fixup ZClass dependencies after '
                          'detecting an updated disk-based product failed.'),
                         error=sys.exc_info())
-                    get_transaction().abort()
+                    transaction.abort()
 
     def install_products(self):
         app = self.getApp()
@@ -587,8 +612,8 @@ def install_products(app):
 
     debug_mode = App.config.getConfiguration().debug_mode
 
-    get_transaction().note('Prior to product installs')
-    get_transaction().commit()
+    transaction.get().note('Prior to product installs')
+    transaction.commit()
 
     products = get_products()
 
@@ -706,7 +731,6 @@ def install_product(app, product_dir, product_name, meta_types,
     path_join=os.path.join
     isdir=os.path.isdir
     exists=os.path.exists
-    DictType=type({})
     global_dict=globals()
     silly=('__doc__',)
 
@@ -727,7 +751,7 @@ def install_product(app, product_dir, product_name, meta_types,
             # like icon images.
             misc_=pgetattr(product, 'misc_', {})
             if misc_:
-                if type(misc_) is DictType:
+                if isinstance(misc_, dict):
                     misc_=Misc_(product_name, misc_)
                 Application.misc_.__dict__[product_name]=misc_
 
@@ -755,6 +779,13 @@ def install_product(app, product_dir, product_name, meta_types,
             # constructors, etc.
             permissions={}
             new_permissions={}
+            if pgetattr(product, '__ac_permissions__', None) is not None:
+                warn('__init__.py of %s has a long deprecated '
+                     '\'__ac_permissions__\' attribute. '
+                     '\'__ac_permissions__\' will be ignored by '
+                     'install_product in Zope 2.10. Please use registerClass '
+                     'instead.' % product.__name__,
+                     DeprecationWarning)
             for p in pgetattr(product, '__ac_permissions__', ()):
                 permission, names, default = (
                     tuple(p)+('Manager',))[:3]
@@ -764,6 +795,12 @@ def install_product(app, product_dir, product_name, meta_types,
                 elif not folder_permissions.has_key(permission):
                     new_permissions[permission]=()
 
+            if pgetattr(product, 'meta_types', None) is not None:
+                warn('__init__.py of %s has a long deprecated \'meta_types\' '
+                     'attribute. \'meta_types\' will be ignored by '
+                     'install_product in Zope 2.10. Please use registerClass '
+                     'instead.' % product.__name__,
+                     DeprecationWarning)
             for meta_type in pgetattr(product, 'meta_types', ()):
                 # Modern product initialization via a ProductContext
                 # adds 'product' and 'permission' keys to the meta_type
@@ -775,6 +812,12 @@ def install_product(app, product_dir, product_name, meta_types,
                 meta_type['visibility'] = 'Global'
                 meta_types.append(meta_type)
 
+            if pgetattr(product, 'methods', None) is not None:
+                warn('__init__.py of %s has a long deprecated \'methods\' '
+                     'attribute. \'methods\' will be ignored by '
+                     'install_product in Zope 2.10. Please use registerClass '
+                     'instead.' % product.__name__,
+                     DeprecationWarning)
             for name,method in pgetattr(
                 product, 'methods', {}).items():
                 if not hasattr(Folder.Folder, name):
@@ -798,16 +841,16 @@ def install_product(app, product_dir, product_name, meta_types,
                     list(Folder.Folder.__ac_permissions__)+new_permissions)
 
             if not doInstall():
-                get_transaction().abort()
+                transaction.abort()
             else:
-                get_transaction().note('Installed product '+product_name)
-                get_transaction().commit()
+                transaction.get().note('Installed product '+product_name)
+                transaction.commit()
 
         except:
             if log_exc:
                 LOG('Zope',ERROR,'Couldn\'t install %s' % product_name,
                     error=sys.exc_info())
-            get_transaction().abort()
+            transaction.abort()
             if raise_exc:
                 raise
 
@@ -844,15 +887,15 @@ def install_standards(app):
         #setattr(Application, fn, ob)
     if wrote:
         app._standard_objects_have_been_added = 1
-        get_transaction().note('Installed standard objects')
-        get_transaction().commit()
+        transaction.get().note('Installed standard objects')
+        transaction.commit()
 
 def reinstall_product(app, product_name):
     folder_permissions = get_folder_permissions()
     meta_types=[]
 
-    get_transaction().note('Prior to product reinstall')
-    get_transaction().commit()
+    transaction.get().note('Prior to product reinstall')
+    transaction.commit()
 
     for product_dir in Products.__path__:
         product_names=os.listdir(product_dir)
@@ -878,9 +921,8 @@ def reimport_product(product_name):
 
 
 def removeProductMetaTypes(pid):
-    '''
-    Unregisters the meta types registered by a product.
-    '''
+    """Unregisters the meta types registered by a product.
+    """
     meta_types = Products.meta_types
     new_mts = []
     changed = 0
